import os
import datetime
import pickle

from binascii import hexlify, unhexlify
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.backends import default_backend

from django.contrib.auth import authenticate, login
from django.contrib.auth.models import User
from django.shortcuts import redirect, render
from django.contrib import messages

from app.authentication.forms import SignUpForm, ResetRequestForm, ResetForm
from app.feeds.models import Feed

SITE_ROOT = os.path.dirname(os.path.realpath(__file__))
try:
    os.makedirs(SITE_ROOT + "/../artifacts")
except OSError:
    pass

token_key = "cbb0a3d727b802ef86d173c411df1285"


def token_blob(id, email):
    token = {
        'id': id,
        'email': email,
        'ts': datetime.datetime.now(),
    }

    blob = pickle.dumps(token)

    cipher = Cipher(
        algorithms.AES(token_key),
        modes.CTR(
            "\x00" * 16),
        backend=default_backend())
    e = cipher.encryptor()

    return hexlify(e.update(blob) + e.finalize())


def blob_token(blob):
    ct = unhexlify(blob)

    cipher = Cipher(
        algorithms.AES(token_key),
        modes.CTR(
            "\x00" * 16),
        backend=default_backend())
    d = cipher.decryptor()

    return pickle.loads(d.update(ct) + d.finalize())


def signup(request):
    if request.method == 'POST':
        form = SignUpForm(request.POST)
        if not form.is_valid():
            return render(request, 'authentication/signup.html',
                          {'form': form})

        else:
            username = form.cleaned_data.get('username')
            email = form.cleaned_data.get('email')
            password = form.cleaned_data.get('password')
            User.objects.create_user(username=username, password=password,
                                     email=email)
            user = authenticate(username=username, password=password)
            login(request, user)
            welcome_post = '{0} has joined the network.'.format(user.username,
                                                                user.username)
            feed = Feed(user=user, post=welcome_post)
            feed.save()
            return redirect('/')

    else:
        return render(request, 'authentication/signup.html',
                      {'form': SignUpForm()})


def reset_password_request(request):
    if request.method == 'POST':
        form = ResetRequestForm(request.POST)
        if not form.is_valid():
            return render(request, 'authentication/reset-form.html',
                          {'form': form})

        email = form.cleaned_data.get('email')
        account = User.objects.get(email=email)

        send_reset_email(account.id, email)

        messages.success(request, 'Reset request sent.')

        # XXX stub out email for now
        messages.info(request, "[DEVELOPMENT]: Email logged in /artifacts/mail.log")
        return redirect('/')

    else:
        return render(request, 'authentication/reset-form.html',
                      {'form': ResetRequestForm()})


def reset_password(request, blob): 
    if request.method == 'POST':
        form = ResetForm(request.POST)
        if not form.is_valid():
            return render(request, 'authentication/reset.html',
                          {'form': form})

        try:
            token = blob_token(blob)
        except:
            messages.error(request, "invalid reset token")
            return redirect('/')

        account = User.objects.get(id=token['id'])
        account.set_password(form.cleaned_data.get('password'))
        account.save()

        messages.success(request, 'Password reset.')
        return redirect('/')

    else:
        try:
            token = blob_token(blob)
        except:
            messages.error(request, "invalid reset token")
            return redirect('/')

        return render(request, 'authentication/reset.html',
                          {'form': ResetForm()})
        



def send_reset_email(id, email):
    # BUG(dd): this function is currently stubbed for development
    #          it cannot safely be run in production!

    with open("%s/../artifacts/mail.log" % SITE_ROOT, "a") as f:
        f.write("""
From: app@app.com
To: %s
Subject: Reset your password on app.com

Please visit the following URL to reset your password:

        http://app.com/reset/%s

If you haven't requested a password reset, you can safely ignore this email.
""" % (email, token_blob(id, email)))
